/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package db;

import domen.Adresa;
import domen.FizickoLice;
import domen.Mesto;
import domen.OpstiDomnskiObjekat;
import domen.Otpremnica;
import domen.PoslovniPartner;
import domen.PravnoLice;
import domen.Proizvod;
import domen.StavkaOtpremnice;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.Date;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;
import konstante.Konstante;

/**
 *
 * @author student1
 */
public class DbKomunikacija {

    private static DbKomunikacija objekat;
    private Connection konekcija;
    String db_url;
    String db_user_name;
    String db_user_password;
    String db_driver_name;

    private DbKomunikacija() throws FileNotFoundException, IOException {

        Properties prop = new Properties();
        prop.load(new FileInputStream(Konstante.TEKUCI_DIREKTORIJUM + File.separator + "config_db.properties"));

        db_url = prop.getProperty("db_url");
        db_user_name = prop.getProperty("db_user_name");
        db_user_password = prop.getProperty("db_user_password");
        db_driver_name = prop.getProperty("db_driver_name");


    }

    public static DbKomunikacija vratiObjekat() throws FileNotFoundException, IOException {
        if (objekat == null) {
            objekat = new DbKomunikacija();
        }
        return objekat;
    }

    public void ucitajDriver() throws ClassNotFoundException {
        //Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
        Class.forName(db_driver_name);
    }

    public void uspostaviKonekciju() throws SQLException {
        //konekcija = DriverManager.getConnection("jdbc:odbc:baza");
        konekcija = DriverManager.getConnection(db_url, db_user_name, db_user_password);
        konekcija.setAutoCommit(false); // Zahteva eksplicitnu potvrdu transakcije
    }

    public void zatvoriKonekciju() throws SQLException {
        konekcija.close();
    }

    public void commitTransakcije() throws Exception {
        try {
            konekcija.commit();
        } catch (SQLException ex) {
            throw new Exception("Greska pri potvrdi transakcije!");
        }
    }

    public void rollbackTransakcije() throws Exception {
        try {
            konekcija.rollback();
        } catch (SQLException ex) {
            throw new Exception("Greska pri ponistenju transakcije!");
        }
    }

    public void sacuvaj(OpstiDomnskiObjekat odo) throws Exception {
        String sql = "INSERT INTO " + odo.vratiImeTabele() + " VALUES " + "(" + odo.vratiVrednostiZaInsert() + ")";
        System.out.println(sql);
        Statement sqlNaredba = konekcija.createStatement();
        sqlNaredba.executeUpdate(sql);
        sqlNaredba.close();
    }

    public List<OpstiDomnskiObjekat> vratiSveObjekte(OpstiDomnskiObjekat odo) throws Exception {
        String sql = "SELECT * FROM " + odo.vratiImeTabele();
        Statement sqlNaredba = konekcija.createStatement();
        ResultSet rs = sqlNaredba.executeQuery(sql);

        return odo.vratiListuIzResultSeta(rs);
    }
//nisu genericke metode

    public void sacuvajMesto(Mesto mesto) throws Exception {
        try {
            String sql = "INSERT INTO MESTO VALUES (?,?)";
            PreparedStatement sqlNaredba = konekcija.prepareStatement(sql);
            //sqlNaredba.setLong(1, mesto.getPtt());
            sqlNaredba.setBigDecimal(1, new BigDecimal(mesto.getPtt()));
            sqlNaredba.setString(2, mesto.getNaziv());
            sqlNaredba.executeUpdate();
            sqlNaredba.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new Exception("Greska pri cuvanju mesta!");
        }
    }

    public List<Mesto> vratiMesta() throws Exception {
        List<Mesto> lm = new ArrayList<Mesto>();
        try {
            String sql = "SELECT * FROM Mesto";
            Statement sqlNaredba = konekcija.createStatement();
            ResultSet mesta = sqlNaredba.executeQuery(sql);
            while (mesta.next()) {
                long ptt = mesta.getLong(1);
                String naziv = mesta.getString("Naziv");
                Mesto m = new Mesto(ptt, naziv);
                lm.add(m);
            }
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new Exception("Greska pri ucitavanju mesta!");
        }
        return lm;
    }

    public void sacuvajFizickoLice(FizickoLice f) throws Exception {
        try {
            String sql = "INSERT INTO PoslovniPartner VALUES (?,?,?,?)";
            PreparedStatement sqlNaredba = konekcija.prepareStatement(sql);
            sqlNaredba.setInt(1, f.getPartnerID());
            sqlNaredba.setString(2, f.getAdresa().getNazivUlice());
            sqlNaredba.setString(3, f.getAdresa().getBroj());
            sqlNaredba.setBigDecimal(4, new BigDecimal(f.getAdresa().getMesto().getPtt()));
            sqlNaredba.executeUpdate();
            sqlNaredba.close();

            sql = "INSERT INTO FizickoLice VALUES (?,?,?,?,?)";
            sqlNaredba = konekcija.prepareStatement(sql);
            sqlNaredba.setInt(1, f.getPartnerID());
            sqlNaredba.setString(2, f.getJmbg());
            sqlNaredba.setString(3, f.getIme());
            sqlNaredba.setString(4, f.getPrezime());
            sqlNaredba.setInt(5, f.getGodRodj());
            sqlNaredba.executeUpdate();
            sqlNaredba.close();

        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new Exception("Greska pri cuvanju fizickog lica!");
        }
    }

    public List<PoslovniPartner> vratiPartnere() throws SQLException {
        List<PoslovniPartner> lp = new ArrayList<PoslovniPartner>();
        String upit = "SELECT * FROM PoslovniPartner";
        Statement s = konekcija.createStatement();
        ResultSet tabela = s.executeQuery(upit);
        while (tabela.next()) {
            int partnerID = tabela.getInt("PartnerID");
            String nazivUlice = tabela.getString("NazivUlice");
            String brojUlice = tabela.getString("BrojUlice");
            long ptt = tabela.getLong("Ptt");
            Mesto m = new Mesto();
            m.setPtt(ptt);
            Adresa a = new Adresa(nazivUlice, brojUlice, m);
            PoslovniPartner pp = vratiPoslovnogPartnera(partnerID);
            pp.setAdresa(a);
            pp.setPartnerID(partnerID);


            lp.add(pp);
        }
        return lp;
    }

    private PoslovniPartner vratiPoslovnogPartnera(long partnerID) throws SQLException {

        PoslovniPartner pp = null;
        String upit = "SELECT * FROM FizickoLice WHERE PartnerID=" + partnerID;
        Statement s = konekcija.createStatement();
        ResultSet tabela = s.executeQuery(upit);
        if (tabela.next()) {

            String jmbg = tabela.getString("JMBG");
            String ime = tabela.getString("Ime");
            String prezime = tabela.getString("Prezime");
            int godRodj = tabela.getInt("GodRodj");
            pp = new FizickoLice(jmbg, ime, prezime, godRodj);
        } else {
            pp = new PravnoLice();
        }
        return pp;
    }

    public List<Proizvod> vratiSveProizvode() throws SQLException {
        String sql = "SELECT * FROM TProizvod";
        Statement s = konekcija.createStatement();
        ResultSet rs = s.executeQuery(sql);
        List<Proizvod> listaP = new ArrayList<Proizvod>();
        while (rs.next()) {
            long sifra = rs.getLong("Sifra");
            String naziv = rs.getString("Naziv");
            double cena = rs.getDouble("Cena");
            long kolicina = rs.getLong("KolicinaNaStanju");
            Proizvod p = new Proizvod(sifra, naziv, cena, kolicina);
            listaP.add(p);
        }
        return listaP;
    }

    public void sacuvajOtpremnicu(Otpremnica o) throws Exception {
        if (o.getListaStavki().isEmpty()) {
            throw new Exception("Otpremnica nema stavke!");
        }

        String sql = "INSERT INTO TOtpremnica VALUES(?,?,?)";
        PreparedStatement ps = konekcija.prepareStatement(sql);
        ps.setBigDecimal(1, new BigDecimal(o.getBrojOtpremnice()));
        ps.setBigDecimal(2, new BigDecimal(o.getPp().getPartnerID()));
        ps.setDate(3, new Date(o.getDatumKreiranja().getTime()));
        ps.executeUpdate();

        List<StavkaOtpremnice> lista = o.getListaStavki();
        String sqlstavka = "INSERT INTO TStavkaOtpremnice VALUES (?,?,?,?,?,?)";
        PreparedStatement prepStat = konekcija.prepareStatement(sqlstavka);

        for (StavkaOtpremnice so : lista) {
            prepStat.setBigDecimal(1, new BigDecimal(o.getBrojOtpremnice()));
            prepStat.setInt(2, so.getRb());
            prepStat.setBigDecimal(3, new BigDecimal(so.getProizvod().getSifra()));
            prepStat.setInt(4, Integer.parseInt(so.getKolicina() + ""));
            prepStat.setDouble(5, so.getCena());
            prepStat.setBigDecimal(6, new BigDecimal(so.getIznosStavke()));

            prepStat.executeUpdate();
        }
    }
}
