/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package db;

import domen.Adresa;
import domen.FizickoLice;
import domen.Mesto;
import domen.Otpremnica;
import domen.PoslovniPartner;
import domen.Proizvod;
import domen.StavkaOtpremnice;
import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.Date;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author student1
 */
public class DbKomunikacija {

    private Connection konekcija;

    public void ucitajDriver() throws ClassNotFoundException {
        Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
    }

    public void uspostaviKonekciju() throws SQLException {
        konekcija = DriverManager.getConnection("jdbc:odbc:D3");
        konekcija.setAutoCommit(false); // Zahteva eksplicitnu potvrdu transakcije
    }

    public void zatvoriKonekciju() throws SQLException {
        konekcija.close();
    }

    public void commitTransakcije() throws Exception {
        try {
            konekcija.commit();
        } catch (SQLException ex) {
            throw new Exception("Greska pri potvrdi transakcije!");
        }
    }

    public void rollbackTransakcije() throws Exception {
        try {
            konekcija.rollback();
        } catch (SQLException ex) {
            throw new Exception("Greska pri ponistenju transakcije!");
        }
    }

    public void sacuvajMesto(Mesto mesto) throws Exception {
        try {
            String sql = "INSERT INTO MESTO VALUES (?,?)";
            PreparedStatement sqlNaredba = konekcija.prepareStatement(sql);
            //sqlNaredba.setLong(1, mesto.getPtt());
            sqlNaredba.setBigDecimal(1, new BigDecimal(mesto.getPtt()));
            sqlNaredba.setString(2, mesto.getNaziv());
            sqlNaredba.executeUpdate();
            sqlNaredba.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new Exception("Greska pri cuvanju mesta!");
        }
    }

    public List<Mesto> vratiMesta() throws Exception {
        List<Mesto> lm = new ArrayList<Mesto>();
        try {
            String sql = "SELECT * FROM Mesto";
            Statement sqlNaredba = konekcija.createStatement();
            ResultSet mesta = sqlNaredba.executeQuery(sql);
            while (mesta.next()) {
                long ptt = mesta.getLong(1);
                String naziv = mesta.getString("Naziv");
                Mesto m = new Mesto(ptt, naziv);
                lm.add(m);
            }
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new Exception("Greska pri ucitavanju mesta!");
        }
        return lm;
    }

    public void sacuvajFizickoLice(FizickoLice f) throws Exception {
        try {
            String sql = "INSERT INTO PoslovniPartner VALUES (?,?,?,?)";
            PreparedStatement sqlNaredba = konekcija.prepareStatement(sql);
            sqlNaredba.setInt(1, f.getPartnerID());
            sqlNaredba.setString(2, f.getAdresa().getNazivUlice());
            sqlNaredba.setString(3, f.getAdresa().getBroj());
            sqlNaredba.setBigDecimal(4, new BigDecimal(f.getAdresa().getMesto().getPtt()));
            sqlNaredba.executeUpdate();
            sqlNaredba.close();

            sql = "INSERT INTO FizickoLice VALUES (?,?,?,?,?)";
            sqlNaredba = konekcija.prepareStatement(sql);
            sqlNaredba.setInt(1, f.getPartnerID());
            sqlNaredba.setString(2, f.getJmbg());
            sqlNaredba.setString(3, f.getIme());
            sqlNaredba.setString(4, f.getPrezime());
            sqlNaredba.setInt(5, f.getGodRodj());
            sqlNaredba.executeUpdate();
            sqlNaredba.close();

        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new Exception("Greska pri cuvanju fizickog lica!");
        }
    }

    public List<PoslovniPartner> vratiListuPoslovnihPartnera() throws Exception {
        String upit = "SELECT * FROM PoslovniPartner";
        Statement s = konekcija.createStatement();
        ResultSet rs = s.executeQuery(upit);
        List<PoslovniPartner> lpp = new ArrayList<PoslovniPartner>();
        while (rs.next()) {
            int partnerId = rs.getInt("PartnerID");
            String nazivUlice = rs.getString("NazivUlice");
            String brojUlice = rs.getString("BrojUlice");
            long ptt = rs.getLong("Ptt");
            Mesto m = new Mesto();
            m.setPtt(ptt);
            //m=vratiMesto(ptt);
            Adresa a = new Adresa(nazivUlice, brojUlice, m);
            PoslovniPartner pp = vratiPoslovnogPartnera(partnerId);
            pp.setAdresa(a);
            pp.setPartnerID(partnerId);
            lpp.add(pp);
        }
        return lpp;
    }

    private PoslovniPartner vratiPoslovnogPartnera(int partnerId) throws Exception {
        String upit = "SELECT * FROM FizickoLice WHERE PartnerID=" + partnerId;
        Statement s = konekcija.createStatement();
        ResultSet rs = s.executeQuery(upit);
        PoslovniPartner pp = null;
        if (rs.next()) {
            int partner = rs.getInt("PartnerID");
            String JMBG = rs.getString("JMBG");
            String ime = rs.getString("Ime");
            String prezime = rs.getString("Prezime");
            int godrodj = rs.getInt("GodRodj");
            pp = new FizickoLice(JMBG, ime, prezime, godrodj);
        } else {
//           //vidi da nije pravno lice
        }
        return pp;
    }

    public List<Proizvod> vratiListuProizvoda() throws SQLException {
        List<Proizvod> lp = new ArrayList<Proizvod>();
        String upit = "SELECT * FROM TPROIZVOD ";
        Statement st = konekcija.createStatement();
        ResultSet rez = st.executeQuery(upit);
        while (rez.next()) {
            long sifra = rez.getLong("Sifra");
            String naziv = rez.getString("Naziv");
            double cena = rez.getDouble("Cena");
            long kolicinaNaStanju = rez.getLong("KolicinaNaStanju");

            Proizvod p = new Proizvod(sifra, naziv, cena, kolicinaNaStanju);
            lp.add(p);
        }
        return lp;
    }

    public void sacuvajOtpremnicu(Otpremnica o) throws SQLException {
        String sdatum = new SimpleDateFormat("dd/MM/yyyy").format(o.getDatumKreiranja());
        String upit = "INSERT INTO TOtpremnica VALUES ("+o.getBrojOtpremnice()+","+o.getPp().getPartnerID()+",#"+sdatum+"#)";
        System.out.println("UPIT: "+upit);
        Statement stat = konekcija.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_READ_ONLY);
        stat.executeUpdate(upit);
        /*PreparedStatement ps = konekcija.prepareStatement(upit);
        ps.setBigDecimal(1, new BigDecimal(o.getBrojOtpremnice()));
        ps.setBigDecimal(2, new BigDecimal(o.getPp().getPartnerID()));
        ps.setDate(3, new Date(o.getDatumKreiranja().getTime()));
        ps.executeUpdate(upit);*/
    }

    public void sacuvajStavkuOtpremnice(StavkaOtpremnice st) throws SQLException {
        //String upit = "INSERT INTO TStavkaOtpremnice VALUES (?,?,?,?,?,?)";
        String upit = "INSERT INTO TStavkaOtpremnice VALUES ("+st.getOtpremnica().getBrojOtpremnice()+","+st.getRb()+","+st.getProizvod().getSifra()+","+st.getKolicina()+","+st.getCena()+","+st.getIznosStavke()+")";
        //Statement stat = konekcija.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_READ_ONLY);
        Statement stat = konekcija.createStatement();
        stat.executeUpdate(upit);
        /*PreparedStatement ps = konekcija.prepareStatement(upit);
        ps.setBigDecimal(1, new BigDecimal(st.getOtpremnica().getBrojOtpremnice()));
        ps.setInt(2, st.getRb());
        ps.setBigDecimal(3, new BigDecimal(st.getProizvod().getSifra()));
        ps.setInt(4, st.getKolicina());
        ps.setDouble(5, st.getCena());
        ps.setBigDecimal(6, new BigDecimal(st.getIznosStavke()));
        ps.executeUpdate(upit);*/
    }

    public long vratiBrojNoveOtpremnice() throws SQLException {
        String upit = "SELECT max(BrojOtpremnice) as MAX FROM TOtpremnica";
        Statement st = konekcija.createStatement(ResultSet.TYPE_SCROLL_SENSITIVE, ResultSet.CONCUR_READ_ONLY);
        ResultSet rs = st.executeQuery(upit);
        long noviBroj = 1;
        if (rs.next()) {
            noviBroj = rs.getLong("MAX")+1;
        }

        return noviBroj;
    }
}
