/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package baza;

import domen.Adresa;
import domen.FizickoLice;
import domen.Mesto;
import domen.Otpremnica;
import domen.PravnoLice;
import domen.Proizvod;
import domen.StavkaOtpremnice;
import java.math.BigDecimal;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author student1
 */
public class DbKomunikacija {

    private Connection connection;

    public void ucitajDriver() throws Exception {
        Class.forName("sun.jdbc.odbc.JdbcOdbcDriver");
    }

    public void otvoriKonekciju() throws Exception {
        connection = DriverManager.getConnection("jdbc:odbc:D1");
        connection.setAutoCommit(false); // Zahteva eksplicitnu potvrdu transakcije
    }

    public void zatvoriKonekciju() throws Exception {
        connection.close();
    }

    public void commitTransakcije() throws Exception {
        try {
            connection.commit();
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new Exception("Transakcija nije potvrdjena!");
        }
    }

    public void rollbackTransakcije() throws Exception {
        try {
            connection.rollback();
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new Exception("Transakcija nije potvrdjena!");
        }
    }

    public void sacuvajMesto(Mesto mesto) throws Exception {
        try {
            String sql = "INSERT INTO Mesto VALUES (?,?)";
            PreparedStatement sqlNaredba = connection.prepareStatement(sql);
            //sqlNaredba.setLong(1, mesto.getPtt());
            sqlNaredba.setBigDecimal(1, new BigDecimal(mesto.getPtt()));
            sqlNaredba.setString(2, mesto.getNaziv());
            sqlNaredba.executeUpdate();
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new Exception("Mesto nije sacuvano!");
        }
    }

    public List<Mesto> vratiMesta() throws Exception {
        List<Mesto> listaMesta = new ArrayList<Mesto>();
        try {
            String sqlUpit = "SELECT * FROM Mesto";
            Statement sqlNaredba = connection.createStatement();
            ResultSet mesta = sqlNaredba.executeQuery(sqlUpit);

            while (mesta.next()) {
                long ptt = mesta.getLong(1);
                String naziv = mesta.getString("naziv");
                Mesto mesto = new Mesto(ptt, naziv);
                listaMesta.add(mesto);
            }
        } catch (Exception ex) {
            ex.printStackTrace();
            throw new Exception("Greska pri ucitavanju mesta");
        }
        return listaMesta;
    }

    public void sacuvajFizickoLice(FizickoLice f) throws Exception {
        try {
            String sql = "INSERT INTO PoslovniPartner VALUES (?,?,?,?)";
            PreparedStatement sqlNaredba = connection.prepareStatement(sql);
            sqlNaredba.setInt(1, f.getPartnerID());
            sqlNaredba.setString(2, f.getAdresa().getNazivUlice());
            sqlNaredba.setString(3, f.getAdresa().getBrojUlice());
            sqlNaredba.setBigDecimal(4, new BigDecimal(f.getAdresa().getMesto().getPtt()));
            sqlNaredba.executeUpdate();
            sqlNaredba.close();

            sql = "INSERT INTO FizickoLice VALUES (?,?,?,?,?)";
            sqlNaredba = connection.prepareStatement(sql);
            sqlNaredba.setInt(1, f.getPartnerID());
            sqlNaredba.setString(2, f.getJmbg());
            sqlNaredba.setString(3, f.getIme());
            sqlNaredba.setString(4, f.getPrezime());
            sqlNaredba.setInt(5, f.getGodRodj());
            sqlNaredba.executeUpdate();
            sqlNaredba.close();

        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new Exception("Fizicko lice nije sacuvano");
        }

    }

    public List<FizickoLice> vratiFizickaLica() throws Exception {
        List<FizickoLice> lista = new ArrayList<FizickoLice>();
        try {
            String sql = "SELECT * FROM FizickoLice f, PoslovniPartner p, Mesto m WHERE f.PartnerID = p.PartnerID AND m.Ptt = p.Ptt";
            Statement statement = connection.createStatement();
            ResultSet fizLica = statement.executeQuery(sql);
            while (fizLica.next()) {
                int partnerID = fizLica.getInt("PartnerID");
                String jmbg = fizLica.getString("JMBG");
                String ime = fizLica.getString("Ime");
                String prezime = fizLica.getString("Prezime");
                int godRodj = fizLica.getInt("godRodj");
                String nazivUlice = fizLica.getString("NazivUlice");
                String brojUlice = fizLica.getString("BrojUlice");
                long ptt = fizLica.getLong("Ptt");
                String nazivMesta = fizLica.getString("Naziv");

                Mesto mesto = new Mesto(ptt, nazivMesta);
                Adresa adresa = new Adresa(nazivUlice, brojUlice, mesto);
                FizickoLice f = new FizickoLice(jmbg, ime, prezime, godRodj, adresa);
                f.setPartnerID(partnerID);
                lista.add(f);
            }

        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new Exception("Greska pri ucitavanju fizickih lica!");
        }
        return lista;
    }

    public List<PravnoLice> vratiPravnaLica() throws Exception {
        List<PravnoLice> lista = new ArrayList<PravnoLice>();
        try {
            String sql = "SELECT * FROM PravnoLice f, PoslovniPartner p, Mesto m WHERE f.PartnerID = p.PartnerID AND m.Ptt = p.Ptt";
            Statement statement = connection.createStatement();
            ResultSet pravnaLica = statement.executeQuery(sql);
            while (pravnaLica.next()) {
                int partnerID = pravnaLica.getInt("PartnerID");
                String nazivPartnera = pravnaLica.getString("NazivPartnera");
                String zr = pravnaLica.getString("Zr");
                String matBroj = pravnaLica.getString("MatBroj");
                String nazivUlice = pravnaLica.getString("NazivUlice");
                String brojUlice = pravnaLica.getString("BrojUlice");
                long ptt = pravnaLica.getLong("Ptt");
                String nazivMesta = pravnaLica.getString("Naziv");

                Mesto mesto = new Mesto(ptt, nazivMesta);
                Adresa adresa = new Adresa(nazivUlice, brojUlice, mesto);
                PravnoLice p = new PravnoLice(nazivPartnera, zr, matBroj, adresa);
                p.setPartnerID(partnerID);
                lista.add(p);
            }

        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new Exception("Greska pri ucitavanju pravnih lica!");
        }
        return lista;
    }

    public List<Proizvod> vratiProizvode() throws Exception {
        List<Proizvod> lp = new ArrayList<Proizvod>();
        try {
            String sql = "SELECT * FROM Proizvod";
            Statement sqlNaredba = connection.createStatement();
            ResultSet rsTabela = sqlNaredba.executeQuery(sql);
            while (rsTabela.next()) {
                int sifraProizvoda = rsTabela.getInt(1);
                String naziv = rsTabela.getString("Naziv");
                double cena = rsTabela.getDouble("Cena");
                int kolicinaNaStanju = rsTabela.getInt("KolicinaNaStanju");
                Proizvod p = new Proizvod(sifraProizvoda, naziv, cena, kolicinaNaStanju);
                lp.add(p);
            }
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new Exception("Greska pri ucitavanju proizvoda!");
        }
        return lp;
    }
    
    public void sacuvajOtpremnicu(Otpremnica otpremnica) throws Exception {
        try {
            String sql = "INSERT INTO Otpremnica VALUES (?,?,?,?)";
            PreparedStatement sqlNaredba = connection.prepareStatement(sql);
            sqlNaredba.setInt(1, otpremnica.getOtpremnicaID());
            sqlNaredba.setDate(2, new java.sql.Date(otpremnica.getDatum().getTime()));
            sqlNaredba.setDouble(3, otpremnica.getUkupanIznos());
            sqlNaredba.setInt(4, otpremnica.getPoslovniPartner().getPartnerID());
            sqlNaredba.executeUpdate();
            sqlNaredba.close();
            
            sql = "INSERT INTO StavkaOtpremnice VALUES (?,?,?,?,?)";
            sqlNaredba = connection.prepareStatement(sql);
            for (StavkaOtpremnice stavka : otpremnica.getListaStavki()) {
                sqlNaredba.setInt(1, otpremnica.getOtpremnicaID());
                sqlNaredba.setInt(2, stavka.getRbStavke());
                sqlNaredba.setInt(3, stavka.getKolicina());
                sqlNaredba.setDouble(4, stavka.getIznos());
                sqlNaredba.setInt(5, stavka.getProizvod().getProizvodID());
                sqlNaredba.executeUpdate();
            }
            sqlNaredba.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
            throw new Exception("Greska pri cuvanju otpremnice!");
        }
    }
}
